@@ -4,7 +4,8 @@ from os.path import join
 from os import unlink
 from arv.lib.util import (valid, try_unique_column, get_keyid_in_certif,
     get_keyid_from_certifstring, trace, bin_encoding, suppress_colon,
-    cred_end_validity_date, escape_special_characters)
+    cred_end_validity_date, escape_special_characters, der_to_pem,
+    split_pkcs7, is_ca)
 from creole import cert
 from arv.config import (x509_default_key_bits, ssl_default_cert_time,
     ca_conf_file, conf_file_orig, ssl_dir)
@@ -53,40 +54,12 @@ def get_tmpl_edges(tmpl_connect):
         raise Exception( 'error in get_tmpl_edges')
 
 @trace()
-def del_tmpl_edge(tmpl_edge):
-    if tmpl_edge.tmpl_connect.connects == []:
-        if tmpl_edge.tmpl_vertex_a == tmpl_edge.tmpl_vertex_b:
-            if (tmpl_edge.tmpl_vertex_a.tmpl_edges_a == [tmpl_edge] \
-                or tmpl_edge.tmpl_vertex_a.tmpl_edges_a == []) \
-                and (tmpl_edge.tmpl_vertex_a.tmpl_edges_b == [tmpl_edge] \
-                or tmpl_edge.tmpl_vertex_a.tmpl_edges_b == []) :
-                # remove vertices and tmpl_vertex_a template
-                for vertex in tmpl_edge.tmpl_vertex_a.vertices:
-                    vertex.delete()
-                tmpl_edge.tmpl_vertex_a.delete()
-        else:
-            if (tmpl_edge.tmpl_vertex_a.tmpl_edges_a == [tmpl_edge] \
-                or tmpl_edge.tmpl_vertex_a.tmpl_edges_a == []) \
-                and (tmpl_edge.tmpl_vertex_a.tmpl_edges_b == [tmpl_edge] \
-                or tmpl_edge.tmpl_vertex_a.tmpl_edges_b == []) :
-                # remove vertices and tmpl_vertex_a template
-                for vertex in tmpl_edge.tmpl_vertex_a.vertices:
-                    vertex.delete()
-                tmpl_edge.tmpl_vertex_a.delete()
-            if (tmpl_edge.tmpl_vertex_b.tmpl_edges_a == [tmpl_edge] \
-                or tmpl_edge.tmpl_vertex_b.tmpl_edges_a == []) \
-                and (tmpl_edge.tmpl_vertex_b.tmpl_edges_b == [tmpl_edge] \
-                or tmpl_edge.tmpl_vertex_b.tmpl_edges_b == []):
-                # remove vertices and tmpl_vertex_b template
-                for vertex in tmpl_edge.tmpl_vertex_b.vertices:
-                    vertex.delete()
-                tmpl_edge.tmpl_vertex_b.delete()
-    else:
-        for connect in tmpl_edge.tmpl_connect.connects:
-            for edge in connect.edges:
-                if edge.tmpl_edge == tmpl_edge:
-                    logger.debug(u"edge {0} between {1} and {2}".format(tmpl_edge.name, connect.head_node.name, connect.tail_node.name))
-                    raise Exception("Can't suppress edge template used by connect !!!")
+def del_tmpl_edge(tmpl_edge, tmpl_connect=None):
+    for connect in tmpl_edge.tmpl_connect.connects:
+        for edge in connect.edges:
+            if edge.tmpl_edge == tmpl_edge:
+                logger.debug(u"edge {0} between {1} and {2}".format(tmpl_edge.name, connect.head_node.name, connect.tail_node.name))
+                raise Exception("Can't suppress edge template used by connect !!!")
     try:
         tmpl_edge.delete()
     except:
@@ -97,7 +70,8 @@ def del_tmpl_vertex(tmpl_vertex):
     if tmpl_vertex.tmpl_edges_a != [] or tmpl_vertex.tmpl_edges_b != []:
         raise Exception('TmplVertex is already used by a TmplEdge, you cannot delete it')
     if tmpl_vertex.vertices != []:
-        raise Exception("TmplVertex is already used by a Vertex, you cannot delete it")
+        for vertex in tmpl_vertex.vertices:
+            vertex.delete()
     try:
         tmpl_vertex.delete()
     except:
@@ -112,6 +86,7 @@ class TmplConnect(Entity):
     tmpl_edges = OneToMany('TmplEdge')
     connects = OneToMany('Connect')
     cred_auth = ManyToOne('Credential')
+    leftsendcert = Field(Text, default='always')
     state = Field(Integer, default=0)
 
     def _add_tmpl_edge(self, name, tmpl_vertex_a, tmpl_vertex_b):
@@ -139,14 +114,19 @@ class TmplConnect(Entity):
 #            raise Exception("error in add_tmpl_edge")
 #        return tmpl
 
-    def add_connect(self, tail_node, head_node, tail_extr, head_extr, tail_cred, head_cred):
+    def add_connect(self, tail_node, head_node, tail_extr, head_extr, tail_cred, head_cred, leftsendcert=None):
+
         try:
             if tail_node == head_node or tail_extr == head_extr:
                 raise Exception( "wrong argument on add_connect")
+            if leftsendcert is not None:
+                final_leftsendcert = leftsendcert
+            else:
+                final_leftsendcert = self.leftsendcert
             return Connect(tmpl_connect=self, tail_node=tail_node,
                     head_node=head_node, tail_extr=tail_extr,
                     head_extr=head_extr, tail_cred=tail_cred,
-                    head_cred=head_cred)
+                    head_cred=head_cred, leftsendcert=final_leftsendcert)
         except:
             raise Exception("error in add_connect")
 
@@ -213,10 +193,51 @@ def add_credential_auth(credential, passwd=None, local=False):
     except Exception, e:
         raise Exception("error in add_cred_auth: %s" % e)
 
+@trace(hide_args=[1], hide_kwargs=['passwd'])
+def add_certificate_authorities(certif, cert_type=None):
+    """Add certificate autorities :
+        - certif is in DER or PEM format
+        - type_cert is 'pkcs7' or 'x509'
+        - Returns last intermediate CA certificate
+    """
+    try:
+        ca = None
+        if cert_type is None:
+            cert_type, certificate = der_to_pem(certif)
+        else:
+            certificate = certif
+        if cert_type == "pkcs7":
+            ca_cred, certificate = split_pkcs7(certificate)
+            for ca_cert in ca_cred:
+                subject = cert.get_subject(cert=ca_cert)
+                name = unicode(subject[1], 'utf-8')
+                suffix_cred = subject[0]
+                suffix_cred = valid(suffix_cred, 'string')
+                subject = bin_encoding("%s/CN=%s"%(suffix_cred, name))
+                ca = Credential.query.filter_by(subject=subject).first()
+                if ca == None:
+                    ca = add_credential_auth(ca_cert)
+        elif cert_type == "x509":
+            if is_ca(certificate):
+                subject = cert.get_subject(cert=certificate)
+                name = unicode(subject[1], 'utf-8')
+                suffix_cred = subject[0]
+                suffix_cred = valid(suffix_cred, 'string')
+                subject = bin_encoding("%s/CN=%s"%(suffix_cred, name))
+                ca = Credential.query.filter_by(subject=subject).first()
+                if ca == None:
+                    ca = add_credential_auth(certificate)
+        else:
+            raise Exception("Unknown certificate type")
+        return ca
+    except Exception,e:
+        raise ValueError(str(e))
+
 @trace()
-def add_tmpl_connect(name, node_a, node_b, cred_auth):
+def add_tmpl_connect(name, node_a, node_b, cred_auth, leftsendcert='always'):
     try:
         name = valid(name, 'string')
+
         if not cred_auth.ca:
             raise Exception('add_tmpl_connect: attribute cred_auth not a credential authority')
     except Exception, e:
@@ -224,7 +245,7 @@ def add_tmpl_connect(name, node_a, node_b, cred_auth):
     if TmplConnect.query.filter_by(name=name).first() != None:
         raise Exception("Name should be unique")
     try:
-        return TmplConnect(name=name, tmpl_node_a=node_a, tmpl_node_b=node_b, cred_auth=cred_auth)
+        return TmplConnect(name=name, tmpl_node_a=node_a, tmpl_node_b=node_b, cred_auth=cred_auth, leftsendcert=leftsendcert)
     except:
         raise Exception("error in add_tmpl_connect")
 
@@ -249,7 +270,7 @@ def del_tmpl_connect(tmpl_connect):
         raise Exception("TmplConnect is already used by a Connect, you cannot delete it")
     try:
         for tmpledge in tmpl_connect.tmpl_edges:
-            del_tmpl_edge(tmpledge)
+            del_tmpl_edge(tmpledge, tmpl_connect)
     except:
         import traceback
         traceback.print_exc()
@@ -296,6 +317,7 @@ class Connect(Entity):
     tail_cred = ManyToOne('Credential')
     head_cred = ManyToOne('Credential')
     tmpl_connect = ManyToOne('TmplConnect', required=True)
+    leftsendcert = Field(Text, default='always')
     edges = OneToMany('Edge')
     state = Field(Integer, default=0)
 
@@ -475,7 +497,8 @@ def add_credential(name, passwd, node, cred_priv_type):
     if cred_priv_type == 'autosigned':
         cert.gen_certif(certfile=certfile,
                         keyfile=keyfile,
-                        del_passwd=False)
+                        del_passwd=False,
+                        regen=True)
 
         fh = open(certfile, 'r')
         credential = fh.read()
@@ -506,7 +529,8 @@ def add_credential(name, passwd, node, cred_priv_type):
         cert.gen_certif(certfile=certfile,
                         keyfile=keyfile,
                         del_passwd=False,
-                        signe_req=False)
+                        signe_req=False,
+                        regen=True)
         fh = open(keyfile, 'r')
         private_key = fh.read()
         fh.close()
