@@ -3,12 +3,14 @@ from IPy import IP
 from os import unlink
 from elixir import *
 from arv.lib.util import valid, try_unique_column, trace, \
-        get_keyid_from_certifstring, bin_encoding, split_pkcs7, \
+        get_keyid_from_certifstring, bin_encoding, \
         decrypt_privkey, get_keyid_from_certifstring, suppress_colon, \
-        valid_priv_and_cred, cred_end_validity_date
+        valid_priv_and_cred, cred_end_validity_date, der_to_pem, \
+        split_pkcs7
 from creole import cert
 from arv.db.edge import get_connected_nodes, TmplConnect, Credential, \
-                        get_credential_auth, add_credential_auth
+                        get_credential_auth, add_credential_auth, \
+                        add_certificate_authorities
 from arv.lib.logger import logger
 from arv.config import ip_sep
 
@@ -223,11 +225,8 @@ class Node(Entity):
     def add_credential(self, private_key, credential=None,
             unsigned_credential=None, cred_auth=None, name=None, passwd=None,
            keyid_cred=None, subjkey=None, old_credential=None):
-        if name == None:
-            subject = cert.get_subject(cert=credential)
-            name = unicode(subject[1], 'utf-8')
-        name = valid(name, 'string')
         subject = cert.get_subject(cert=credential)
+        name = unicode(subject[1], 'utf-8')
         suffix_cred = subject[0]
         suffix_cred = valid(suffix_cred, 'string')
         subject = bin_encoding("%s/CN=%s"%(suffix_cred, name))
@@ -261,23 +260,15 @@ class Node(Entity):
     @trace(hide_args=[3], hide_kwargs=['passwd'])
     def import_credential(self, private_key, credential, passwd,
             old_credential=None, old_ca=None):
-        ca_cred = None
         ca = None
-        try:
-            ca_cred, credential = split_pkcs7(credential)
-            if ca_cred != None:
-                pkcs7_subject = cert.get_subject(cert=ca_cred)
-                name = unicode(pkcs7_subject[1], 'utf-8')
-                suffix_cred = pkcs7_subject[0]
-                suffix_cred = valid(suffix_cred, 'string')
-                pkcs7_subject = bin_encoding("%s/CN=%s"%(suffix_cred, name))
-                ca = Credential.query.filter_by(subject=pkcs7_subject).first()
-                if ca == None and old_ca is None:
-                    ca = add_credential_auth(ca_cred)
-        except:
-            #file not a pkcs7 file
+        cert_type, certificate = der_to_pem(credential)
+        if old_ca is None:
+            ca = add_certificate_authorities(certificate, cert_type)
+        if cert_type == "pkcs7":
+            ca_cred, certificate = split_pkcs7(certificate)
+        if certificate is not None:
             try:
-                cred_subject = cert.get_issuer_subject(cert=credential)
+                cred_subject = cert.get_issuer_subject(cert=certificate)
                 name = unicode(cred_subject[1], 'utf-8')
                 suffix_cred = cred_subject[0]
                 suffix_cred = valid(suffix_cred, 'string')
@@ -287,6 +278,8 @@ class Node(Entity):
                     raise Exception("CA does not exists in database, Can't add certificate")
             except Exception, e:
                 raise ValueError(str(e))
+        else:
+            raise Exception("Unknown certificate type")
         if old_ca is not None and old_ca != ca:
             raise ValueError("New credential is issued with another CA than old one's")
         subjkey, keyid_cred = get_keyid_from_certifstring(private_key, passwd=passwd)
@@ -294,7 +287,7 @@ class Node(Entity):
         subjkey = suppress_colon(subjkey)
         keyid_cred = suppress_colon(keyid_cred)
         return self.add_credential(private_key=private_key,
-                credential=credential, cred_auth=ca, passwd=passwd,
+                credential=certificate, cred_auth=ca, passwd=passwd,
                 keyid_cred=keyid_cred, subjkey=subjkey,
                 old_credential=old_credential)
 
