@@ -6,7 +6,9 @@ from os.path import isfile, isdir, basename, dirname, join
 from shutil import copy
 import re
 from arv.lib.util import trace, bin_encoding, ipsec_running, ipsec_restart, \
-                         md5, gen_archive_name, normalize_unicode, remove_special_characters
+                         md5, gen_archive_name, normalize_unicode, \
+                         remove_special_characters, extract_CN_from_certifstring
+
 from arv.lib.logger import logger
 from arv.db import initialize as arvdb
 from arv.db.edge import get_credentials_auth, Crl, get_connects
@@ -179,7 +181,7 @@ def fill_database(current_node, arv_node=False):
 
         sw_join_certificate_identity(identityid=type_eleven_key_id, certid=cred.id)
         sw_join_certificate_identity(identityid=type_nine, certid=cred.id)
-    peername_suffix = 1
+    dict_peername_suffix = {}
     for connect in get_connects(nodea=current_node):
         logger.debug("# iterate over get_connects() {0}".format(connect.tmpl_connect.name))
         if connect.tail_node == current_node or connect.head_node == current_node:
@@ -197,7 +199,7 @@ def fill_database(current_node, arv_node=False):
                 credb = connect.head_cred
                 local = connect.tail_extr.priv_ip
                 remote = connect.head_extr.pub_ip
-                name_peercfg = '%s-%s_%d' % (nodea.name, nodeb.name, peername_suffix)
+                name_peercfg = '%s-%s_' % (nodea.name, nodeb.name)
             else:
                 nodea = connect.head_node
                 creda = connect.head_cred
@@ -205,7 +207,12 @@ def fill_database(current_node, arv_node=False):
                 credb = connect.tail_cred
                 local = connect.head_extr.priv_ip
                 remote = connect.tail_extr.pub_ip
-                name_peercfg = '%s-%s_%d' % (nodeb.name, nodea.name, peername_suffix)
+                name_peercfg = '%s-%s_' % (nodeb.name, nodea.name)
+            if name_peercfg in dict_peername_suffix:
+                dict_peername_suffix[name_peercfg] += 1
+            else:
+                dict_peername_suffix[name_peercfg] = 1
+            name_peercfg += str(dict_peername_suffix[name_peercfg])
 
             if creda == None:
                 logger.debug("No credential for {0} in connect {1}".format(unicode.encode(nodea.name, 'utf-8'), unicode.encode(connect.tmpl_connect.name, 'utf8')))
@@ -225,7 +232,6 @@ def fill_database(current_node, arv_node=False):
             #peer_id = local_id
             sw_create_peer_configs(name=name_peercfg, id=ikeid, ike_cfg=ikeid,
                                    local_id=local_id, remote_id=remote_id, virtual=None, pool=None)
-            peername_suffix += 1
             #get all edges for this connect
             logger.debug('edges : {0}'.format(str(connect.edges)))
             for edge in connect.edges:
@@ -389,10 +395,15 @@ def ipsec_conf_build(zephir, current_node, arv_node=False):
         ipsec_conf_content.append("#configuration is in ipsec.conf file")
         ipsec_conf_content.append(ipsec_conf_setup)
 
+        config_md5sum_file = str(uai) + "-" + unicode.encode(name, "utf-8") + ".md5sum"
+        config_md5sum_file = join(node_path, config_md5sum_file)
+        config_md5sum = []
+
         for cred in get_credentials_auth():
             logger.debug("# iterate over get_credentials_auth()")
             #Add CA in ipsec.conf file (optional, only if crl)
             crls = Crl.query.filter_by(ca=cred).all()
+            cred_filename = re.sub('(.*)/(.*)', r'\1_\2', cred.name) + ".pem"
             if crls:
                 uri_number = 0
                 crluri = ''
@@ -405,14 +416,15 @@ def ipsec_conf_build(zephir, current_node, arv_node=False):
                     uri_number +=  1
 
                 ipsec_conf_content.append(ipsec_conf_ca.format(cred.name+"_CA", \
-                                                               cred.name+".pem", \
+                                                               cred_filename, \
                                                                crluri, \
                                                                crluri1))
             # write ca file
-            ca_filename = join(tmpconfigs_directory_cacerts, cred.name+".pem")
+            ca_filename = join(tmpconfigs_directory_cacerts, cred_filename)
             ca_handler = open (ca_filename, "w")
             ca_handler.write(cred.credential)
             ca_handler.close()
+            config_md5sum.append(md5(ca_filename, sw_database_mode='False'))
 
         # generate proposal config for ike and esp
         ike_proposals = []
@@ -448,56 +460,66 @@ def ipsec_conf_build(zephir, current_node, arv_node=False):
                                   sw_mobike))
 
         # Generate connections
-        peername_suffix = 1
+        dict_peername_suffix = {}
         secrets = []
         for connect in get_connects(nodea=current_node):
             logger.debug("# iterate over get_connects() {0}".format(connect.tmpl_connect.name))
-            if connect.tail_node == current_node or connect.head_node == current_node:
+            # Flag for the ordering network into a tunnel
+            if connect.head_node == current_node or connect.tail_node == current_node:
                 logger.debug("#has current_node")
-                ikeid = connect.id
-
-                #search local node
-                #search remote node
-                if connect.tail_node == current_node:
-                    nodea = connect.tail_node
-                    creda = connect.tail_cred
-                    nodeb = connect.head_node
-                    credb = connect.head_cred
-                    local = connect.tail_extr.priv_ip
-                    remote = connect.head_extr.pub_ip
-                    name_peercfg = '%s-%s_%d' % (remove_special_characters(nodea.name), remove_special_characters(nodeb.name), peername_suffix)
+                current_is_head = True if connect.head_node == current_node else False
+                local_node = connect.head_node if current_is_head else connect.tail_node
+                remote_node = connect.tail_node if current_is_head else connect.head_node
+                local_cred = connect.head_cred if current_is_head else connect.tail_cred
+                remote_cred = connect.tail_cred if current_is_head else connect.head_cred
+                local = connect.head_extr.priv_ip if current_is_head else connect.tail_extr.priv_ip
+                remote = connect.tail_extr.pub_ip if current_is_head else connect.head_extr.pub_ip
+                logger.debug(u"Current/local node {0} is {1} node".format(current_node.name, "HEAD" if current_is_head else "TAIL"))
+                logger.debug(u"remote node {0} is {1} node".format(remote_node.name, "TAIL" if current_is_head else "HEAD"))
+                name_peercfg = '%s-%s_' % (remove_special_characters(local_node.name), remove_special_characters(remote_node.name))
+                if name_peercfg in dict_peername_suffix:
+                    dict_peername_suffix[name_peercfg] += 1
                 else:
-                    nodea = connect.head_node
-                    creda = connect.head_cred
-                    nodeb = connect.tail_node
-                    credb = connect.tail_cred
-                    local = connect.head_extr.priv_ip
-                    remote = connect.tail_extr.pub_ip
-                    name_peercfg = '%s-%s_%d' % (remove_special_characters(nodeb.name), remove_special_characters(nodea.name), peername_suffix)
+                    dict_peername_suffix[name_peercfg] = 1
+                name_peercfg += str(dict_peername_suffix[name_peercfg])
 
-                if creda == None:
-                    logger.debug("No credential for {0} in connect {1}".format(unicode.encode(nodea.name, 'utf-8'), unicode.encode(connect.tmpl_connect.name, 'utf8')))
+                if local_cred == None:
+                    logger.debug("No credential for {0} in connect {1}".format(unicode.encode(local_node.name, 'utf-8'), unicode.encode(connect.tmpl_connect.name, 'utf8')))
                     continue
-                if credb == None:
-                    logger.debug("No credential for {0} in connect {1}".format(unicode.encode(nodeb.name, 'utf-8'), unicode.encode(connect.tmpl_connect.name, 'utf-8')))
+                if remote_cred == None:
+                    logger.debug("No credential for {0} in connect {1}".format(unicode.encode(remote_node.name, 'utf-8'), unicode.encode(connect.tmpl_connect.name, 'utf-8')))
                     continue
 
                 # generate certificate and private key files
-                cred_filename = join(tmpconfigs_directory_certs, creda.name + ".pem")
+                local_cred_filename = re.sub('(.*)/(.*)', r'\1_\2', local_cred.name) + ".pem"
+                cred_filename = join(tmpconfigs_directory_certs, local_cred_filename)
+                logger.debug("local_cred_filename %s" % local_cred_filename)
+                local_CN = extract_CN_from_certifstring(local_cred.credential)
+                remote_CN = extract_CN_from_certifstring(remote_cred.credential)
                 cred_handler = open(cred_filename, 'w')
-                cred_handler.write(creda.credential)
+                cred_handler.write(local_cred.credential)
                 cred_handler.close()
-                privkey_filename = join(tmpconfigs_directory_private, "priv" + creda.name + ".pem")
+                config_md5sum.append(md5(cred_filename, sw_database_mode='False'))
+                privkey_filename = join(tmpconfigs_directory_private, "priv" + local_cred_filename)
                 privkey_handler = open(privkey_filename, 'w')
-                privkey_handler.write(creda.private_key)
+                privkey_handler.write(local_cred.private_key)
                 privkey_handler.close()
+                config_md5sum.append(md5(privkey_filename, sw_database_mode='False'))
+                if connect.leftsendcert == 'never':
+                    remote_cred_filename = re.sub('(.*)/(.*)', r'\1_\2', remote_cred.name) + ".pem"
+                    cred_filename = join(tmpconfigs_directory_certs, remote_cred_filename)
+                    cred_handler = open(cred_filename, 'w')
+                    cred_handler.write(remote_cred.credential)
+                    cred_handler.close()
+                    config_md5sum.append(md5(cred_filename, sw_database_mode='False'))
+                else:
+                    remote_cred_filename = ""
 
                 # No duplicate line in ipsec.secrets file
-                if creda.name not in secrets:
-                    secrets.append(creda.name)
-                    ipsec_secrets_content.append(ipsec_secrets.format(creda.suffix_cred+"/CN="+creda.name, \
-                                                                  "priv"+creda.name+".pem"))
-                peername_suffix += 1
+                if local_cred.name not in secrets:
+                    secrets.append(local_cred.name)
+                    ipsec_secrets_content.append(ipsec_secrets.format(local_cred.suffix_cred+"/CN="+local_CN, \
+                                                                  "priv"+local_cred_filename))
                 #get all edges for this connect
                 logger.debug('edges : {0}'.format(str(connect.edges)))
                 for edge in connect.edges:
@@ -507,78 +529,81 @@ def ipsec_conf_build(zephir, current_node, arv_node=False):
                         updown = ''
                     else:
                         updown = '/etc/ipsec.d/ipsec_updown'
-
-                    tmplvertex_a = edge.tmpl_edge.tmpl_vertex_a
-                    tmplvertex_b = edge.tmpl_edge.tmpl_vertex_b
-
-                    if nodea.tmpl_node == tmplvertex_a.tmpl_node:
-                        if  nodeb.tmpl_node != tmplvertex_b.tmpl_node:
-                            logger.error('no relation between nodeb et tmplvertex_b')
-                            raise Exception('no relation between nodeb et tmplvertex_b')
-                        vertex_a = get_vertex_by_node_tmplvertex(node=nodea, tmpl_vertex=tmplvertex_a)
-                        vertex_b = get_vertex_by_node_tmplvertex(node=nodeb, tmpl_vertex=tmplvertex_b)
-                        tmplvertex_name_a = tmplvertex_a.name
-                        tmplvertex_name_b = tmplvertex_b.name
-                    elif nodea.tmpl_node == tmplvertex_b.tmpl_node:
-                        if  nodeb.tmpl_node != tmplvertex_a.tmpl_node:
-                            logger.error('no relation between nodeb et tmplvertex_a')
-                            raise Exception('no relation between nodeb et tmplvertex_a')
-                        vertex_a = get_vertex_by_node_tmplvertex(node=nodea, tmpl_vertex=tmplvertex_b)
-                        vertex_b = get_vertex_by_node_tmplvertex(node=nodeb, tmpl_vertex=tmplvertex_a)
-                        tmplvertex_name_a = tmplvertex_b.name
-                        tmplvertex_name_b = tmplvertex_a.name
+                    if local_node.tmpl_node == remote_node.tmpl_node:
+                        # Connect between same type nodes
+                        logger.debug("Same tmpl_node")
+                        local_tmplvertex = edge.tmpl_edge.tmpl_vertex_a if current_is_head else edge.tmpl_edge.tmpl_vertex_b
+                        remote_tmplvertex = edge.tmpl_edge.tmpl_vertex_b if current_is_head else edge.tmpl_edge.tmpl_vertex_a
                     else:
-                        logger.error('no relation between nodea et tmplvertex')
-                        raise Exception('no relation between nodea et tmplvertex')
-
-                    if vertex_a == None:
-                        logger.debug("No vertex for node %s and tmpl_vertex %s"%(nodea.name, tmplvertex_name_a))
+                        # Connect between different type nodes
+                        logger.debug("different tmpl_node")
+                        if edge.tmpl_edge.tmpl_vertex_a.tmpl_node == local_node.tmpl_node:
+                            local_tmplvertex = edge.tmpl_edge.tmpl_vertex_a
+                            remote_tmplvertex = edge.tmpl_edge.tmpl_vertex_b
+                        else:
+                            local_tmplvertex = edge.tmpl_edge.tmpl_vertex_b
+                            remote_tmplvertex = edge.tmpl_edge.tmpl_vertex_a
+                    logger.debug(u"local tmplvertex {0}".format(local_tmplvertex.name))
+                    logger.debug(u"remote tmplvertex {0}".format(remote_tmplvertex.name))
+                    local_vertex = get_vertex_by_node_tmplvertex(node=local_node, tmpl_vertex=local_tmplvertex)
+                    remote_vertex = get_vertex_by_node_tmplvertex(node=remote_node, tmpl_vertex=remote_tmplvertex)
+                    if local_vertex == None:
+                        logger.debug("No vertex for node %s and tmpl_vertex %s"%(local_node.name, local_tmplvertex.name))
                         continue
-                    if vertex_b == None:
-                        logger.debug("No vertex for node %s and tmpl_vertex %s"%(nodeb.name, tmplvertex_name_b))
+                    if remote_vertex == None:
+                        logger.debug("No vertex for node %s and tmpl_vertex %s"%(remote_node.name, remote_tmplvertex.name))
                         continue
 
                     # generate connection
-                    leftid = re.sub(r'/', ', ', re.sub(r'^/', '', creda.suffix_cred)) + ", CN=" + creda.name
-                    rightid = re.sub(r'/', ', ', re.sub(r'^/', '', credb.suffix_cred)) + ", CN=" + credb.name
-                    if vertex_a.tmpl_vertex.mimetype == u'ip':
-                        leftsubnet = str(IP(vertex_a.ip1+"/255.255.255.255"))
-                    elif vertex_a.tmpl_vertex.mimetype == u'network':
-                        if ip_sep in vertex_a.ip1:
-                            vip1 = vertex_a.ip1.split(ip_sep)
-                            vip2 = vertex_a.ip2.split(ip_sep)
+                    leftid = re.sub(r'/', ', ', re.sub(r'^/', '', local_cred.suffix_cred)) + ", CN=" + local_CN
+                    rightid = re.sub(r'/', ', ', re.sub(r'^/', '', remote_cred.suffix_cred)) + ", CN=" + remote_CN
+                    if local_vertex.tmpl_vertex.mimetype == u'ip':
+                        leftsubnet = str(IP(local_vertex.ip1))+"/32"
+                        logger.debug("LeftSubnet IP : {0}".format(leftsubnet))
+                    elif local_vertex.tmpl_vertex.mimetype == u'network':
+                        if ip_sep in local_vertex.ip1:
+                            vip1 = local_vertex.ip1.split(ip_sep)
+                            vip2 = local_vertex.ip2.split(ip_sep)
                             leftsubnet = []
                             for i in range(0, len(vip1)):
                                 leftsubnet.append(str(IP('%s/%s'%(vip1[i], vip2[i]))))
                             leftsubnet = ','.join(leftsubnet)
                         else:
-                            leftsubnet = str(IP(vertex_a.ip1+"/"+vertex_a.ip2))
-                    elif vertex_a.tmpl_vertex.mimetype == u'range':
+                            leftsubnet = str(IP(local_vertex.ip1+"/"+local_vertex.ip2))
+                    elif local_vertex.tmpl_vertex.mimetype == u'range':
                         raise Exception("Range network not supported")
-                    if vertex_b.tmpl_vertex.mimetype == u'ip':
-                        rightsubnet = str(IP(vertex_b.ip1+"/255.255.255.255"))
-                    elif vertex_b.tmpl_vertex.mimetype == u'network':
-                        if ip_sep in vertex_b.ip1:
-                            vip1 = vertex_b.ip1.split(ip_sep)
-                            vip2 = vertex_b.ip2.split(ip_sep)
+                    if remote_vertex.tmpl_vertex.mimetype == u'ip':
+                        rightsubnet = str(IP(remote_vertex.ip1))+"/32"
+                        logger.debug("RightSubnet IP : {0}".format(rightsubnet))
+                    elif remote_vertex.tmpl_vertex.mimetype == u'network':
+                        if ip_sep in remote_vertex.ip1:
+                            vip1 = remote_vertex.ip1.split(ip_sep)
+                            vip2 = remote_vertex.ip2.split(ip_sep)
                             rightsubnet = []
                             for i in range(0, len(vip1)):
                                 rightsubnet.append(str(IP('%s/%s'%(vip1[i], vip2[i]))))
                             rightsubnet = ','.join(rightsubnet)
                         else:
-                            rightsubnet = str(IP(vertex_b.ip1+"/"+vertex_b.ip2))
-                    elif vertex_b.tmpl_vertex.mimetype == u'range':
+                            rightsubnet = str(IP(remote_vertex.ip1+"/"+remote_vertex.ip2))
+                    elif remote_vertex.tmpl_vertex.mimetype == u'range':
                         raise Exception("Range network not supported")
-                    ipsec_conf_content.append(ipsec_conf_connection.format(\
-                                              connection_name, \
-                                              leftid, \
-                                              creda.name+".pem", \
-                                              local, \
-                                              leftsubnet, \
-                                              updown, \
-                                              rightid, \
-                                              remote, \
-                                              rightsubnet))
+                    ipsec_conn = ipsec_conf_connection.format(\
+                        connection_name, \
+                        leftid, \
+                        local_cred_filename, \
+                        connect.leftsendcert, \
+                        local, \
+                        leftsubnet, \
+                        updown, \
+                        rightid, \
+                        remote_cred_filename, \
+                        remote, \
+                        rightsubnet)
+                    if connect.leftsendcert == 'always':
+                        # Suppress unnecessary config lines
+                        ipsec_conn = re.sub(r'.*leftsendcert = .*\n', r'', ipsec_conn)
+                        ipsec_conn = re.sub(r'.*rightcert = .*\n', r'', ipsec_conn)
+                    ipsec_conf_content.append(ipsec_conn)
             else:
                 logger.debug("no %s node for %s" % (current_node.name, connect.tmpl_connect.name))
         # END generate connections
@@ -588,19 +613,16 @@ def ipsec_conf_build(zephir, current_node, arv_node=False):
         ipsec_conf_handler = open(tmpconfigs_ipsec_conf_filename, "w")
         ipsec_conf_handler.write(unicode.encode(ipsec_conf_content, "utf-8"))
         ipsec_conf_handler.close()
+        config_md5sum.append(md5(tmpconfigs_ipsec_conf_filename, sw_database_mode='False'))
 
         # write ipsec.secrets file with ipsec_secrets_content
         ipsec_secrets_content = '\n'.join(ipsec_secrets_content)
         ipsec_secrets_handler = open(tmpconfigs_ipsec_secrets_filename, "w")
         ipsec_secrets_handler.write(ipsec_secrets_content)
         ipsec_secrets_handler.close()
+        config_md5sum.append(md5(tmpconfigs_ipsec_secrets_filename, sw_database_mode='False'))
 
         logger.debug("Creating VPN archive for {0} {1} server.".format(uai, unicode.encode(name, "utf-8")))
-        config_md5sum_file = str(uai) + "-" + unicode.encode(name, "utf-8") + ".md5sum"
-        config_md5sum_file = join(node_path, config_md5sum_file)
-        config_md5sum = []
-        config_md5sum.append(md5(tmpconfigs_ipsec_conf_filename, sw_database_mode='False'))
-
         # write test-rvp file
         test_rvp_filename = join(strongswan_tmpconfigs_directory, "test-rvp")
         ftest_rvp = open(test_rvp_filename, "w")
