@@ -3,19 +3,26 @@ import logging
 import re
 import hashlib
 import time
+import binascii
 from tempfile import mkstemp
-from os import unlink, system, sep, listdir
-from os.path import basename, dirname, isdir, join
+from os import unlink, system, sep, listdir, close, open as osopen, fdopen, O_WRONLY, O_CREAT
+from os.path import basename, dirname, isdir, join, exists
 from commands import getoutput, getstatusoutput
+from pyeole.process import system_out
+import subprocess
 
-from creole.cert import ca_keyfile
+from creole import cert
 
 from formencode import validators, foreach
 from sqlalchemy.exc import IntegrityError
-from arv.config import key2keyid_path, id2sql_path, bin2sql_path, ssl_dir, \
-                       vpn_path
+#from arv.config import id2sql_path
+#from arv.config import bin2sql_path
+from arv.config import ssl_dir
+from arv.config import vpn_path
 from arv.lib.logger import logger
 
+cert.load_default_conf_if_needed()
+
 def trace(hide_args=None, hide_kwargs=None):
     """This is a decorator which can be used to trace functions calls
 
@@ -141,8 +148,10 @@ def get_lastline(lines):
 def bin_encoding(clear):
     """encode in binary for strongswan database
     """
-    cmd = '%s "%s"' % (id2sql_path, clear)
-    result = get_scndline(getoutput(cmd))
+    # WARNING : THIS DOESN'T REPLACE ID2SQL SCRIPT.
+    # THIS IS JUST TO MAKE ARV DATABASE BUILDNING POSSIBLE WITHOUT ID2SQL
+    # STRONGSWAN DATABASE MODE SHOULDN'T WORKS WITH THAT
+    result = binascii.hexlify(clear)
     #encode and decode. this functionality already exists with the encodings library (which is built-in)
     # have to decode to put the normal string in a sqlalchemy binary type
     logger.debug("#-> bin_encoding result var: " + result)
@@ -164,7 +173,7 @@ def get_keyid_in_certif(certif_name, passwd=None, certiftype='ca'):
     certif_path = dirname(certif_path)
     certif_path = join(certif_path, 'private')
     if certiftype == 'ca':
-        priv_certname = ca_keyfile
+        priv_certname = cert.ca_keyfile
     else:
         priv_certname = join(certif_path, 'priv-'+certif_name)
     return get_keyid_from_keyid_in_certif(priv_certname, passwd, mode='rsa')
@@ -176,8 +185,22 @@ def get_keyid_from_keyid_in_certif(certif_name, passwd=None, mode='rsa'):
     if mode == 'rsa':
         if password_OK(certif_name, passwd):
             #Extract from private key
-            cmd = 'openssl {0} -in {1} -text -passin pass:"{2}" | {3}'.format(mode, certif_name, passwd, key2keyid_path)
-            subjkey, keyid = get_lastline(getoutput(cmd))
+            sha1_cmd = ['openssl', 'sha1', '-c']
+            rsa_cmd = ['openssl', 'rsa', '-in', certif_name,
+                       '-passin', 'stdin', '-outform', 'DER']
+
+            pub_info_cmd = rsa_cmd + ['-pubout']
+            code, pub_info_der, stderr = system_out(pub_info_cmd, stdin=passwd)
+
+            pub_key_cmd = rsa_cmd + ['-RSAPublicKey_out']
+            code, pub_key_der, stderr = system_out(pub_key_cmd, stdin=passwd)
+
+            code, pub_info_fpr, stderr = system_out(sha1_cmd, pub_info_der)
+            code, pub_key_fpr, stderr = system_out(sha1_cmd, pub_key_der)
+
+            subjkey = pub_info_fpr.split()[-1].strip()
+            keyid = pub_key_fpr.split()[-1].strip()
+            return (subjkey, keyid)
         else:
             raise Exception('Invalid password')
     elif mode == 'x509':
@@ -199,6 +222,7 @@ def get_keyid_from_certifstring(certif_string, passwd=None, mode='rsa'):
         fh.write(certif_string)
         fh.close()
         subjkey, keyid = get_keyid_from_keyid_in_certif(certif_name, passwd, mode=mode)
+        close(fd)
         unlink(certif_name)
         return (subjkey, keyid)
     except Exception, e:
@@ -257,55 +281,164 @@ def gen_archive_name(uai, name):
     archivename = str(uai) + "-" + unicode.encode(name, "utf-8") + ".tar.gz"
     return amonpath, archivename
 
+@trace()
+def der_to_pem(certificate):
+    """
+    Convert certificate to PEM format
+    Returns tuple : (cert_type, pem_cert)
+    """
+    fd, certif_filename = mkstemp()
+    fh = file(certif_filename, 'w')
+    fh.write(certificate)
+    fh.close()
+    close(fd)
+    cert_types = ["x509", "pkcs7"]
+    cert_formats = ["DER", "PEM"]
+    for cert_type in cert_types:
+        for cert_format in cert_formats:
+            cmd = 'openssl {0} -inform {1} -in {2}'.format(cert_type, cert_format, certif_filename).split()
+            process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
+            ret_code = process.wait()
+            if ret_code == 0:
+                unlink(certif_filename)
+                return cert_type, process.stdout.read()
+    unlink(certif_filename)
+    return None, None
+
+@trace()
+def is_ca(certificate):
+    """
+    certificate is in PEM format
+    Returns True if certificate is a CA, False if not
+    """
+    cmd = "openssl x509 -noout -text".split()
+    process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
+    process.stdin.write(certificate)
+    ret_code = process.wait()
+    if ret_code != 0:
+        raise Exception("Error in execution 'openssl x509' command")
+    text_cert = process.stdout.read().splitlines()
+    for line in text_cert:
+        if re.match(r'(.*)CA:TRUE(.*)', line):
+            return True
+    return False
+
 @trace()
 def split_pkcs7(pkcs7_cred):
     """Split pkcs7 string in two strings ca_cred and cred
+        returns :
+         * ca certificates into a string list in PEM format
+           sorted first root CA certificate to intermediates CAs
+         * end-user certificate
     """
     try:
-        cmd = 'echo "{0}" | openssl pkcs7 -print_certs '.format(pkcs7_cred)
-        two_creds = getoutput(cmd)
-        # = content.slpit('\n')
-        new_cred = True
-        issuer = ''
-        ca_cred = ''
+        cmd = 'openssl pkcs7 -print_certs'.split()
+        process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
+        process.stdin.write(pkcs7_cred)
+        ret_code = process.wait()
+        if ret_code != 0:
+            raise Exception("Error in execution 'openssl pkcs7' command")
+        creds_chain = process.stdout.read()
+        # Append cert_list
+        append_cert_line = False
+        new_cert = True
+        cert_list = []
+        index = 0
+        for line in creds_chain.splitlines():
+            if line == '-----BEGIN CERTIFICATE-----':
+                append_cert_line = True
+                new_cert = True
+            if append_cert_line:
+                if new_cert:
+                    cert_list.append('{0}\n'.format(line))
+                    new_cert = False
+                else:
+                    cert_list[index] += '{0}\n'.format(line)
+            if line == '-----END CERTIFICATE-----':
+                append_cert_line = False
+                index +=1
+        # Filter certificates :
+        #  - CAs in ca_certs dict
+        #  - end-user cerificate in cred
+        ca_certs= {}
         cred = ''
-        for line in two_creds.splitlines():
-            if line == '':
-                subject = ''
-                issuer = ''
-                new_cred = True
-            elif new_cred:
-                subject = re.match( r'subject(.*)CN=(.*)', line)
-                if not subject:
-                    raise Exception('Cannot get subject from pkcs7 file')
-                subject = subject.group(2)
-                new_cred = False
-            elif not new_cred and issuer == '' and subject:
-                issuer = re.match( r'issuer(.*)CN=(.*)', line)
-                if not issuer:
-                    raise Exception('Cannot get issuer from pkcs7 file')
-                issuer = issuer.group(2)
-                if subject == issuer and subject != '':
-                    if ca_cred != '':
-                        raise Exception('Already a ca_cred in pkcs7 file')
-                else:
-                    if cred != '':
-                        raise Exception('Already a cred in pkcs7 file')
-            elif subject and issuer:
-                if subject == issuer:
-                    ca_cred += '{0}\n'.format(line)
-                else:
-                    cred += '{0}\n'.format(line)
+        for certif in cert_list:
+            if is_ca(certif):
+                cacert_subject = cert.get_subject(certif)[1]
+                if cacert_subject not in ca_certs:
+                    ca_certs[cacert_subject] = certif
             else:
-                raise Exception('Unknown line in pkcs7 file {0}'.format(line))
-
-        if ca_cred == '' or cred == '':
-            raise Exception('Pkcs7 file not complete')
-        return (ca_cred, cred)
+                cred = certif
+        # Get root ca certificate subject
+        ca_subject = None
+        issuer_ca_subject = None
+        for ca_subject, ca_cert in ca_certs.items():
+            issuer_ca_subject = cert.get_issuer_subject(ca_cert)[1]
+            if ca_subject == issuer_ca_subject:
+                break
+        # Sort CA certificates
+        ca_creds = []
+        while True and ca_certs != {}:
+            ca_creds.append(ca_certs.get(ca_subject))
+            issuer_ca_subject = ca_subject
+            ca_subject = next((ca_subj for ca_subj, ca_cert in ca_certs.items() if cert.get_issuer_subject(ca_cert)[1] == issuer_ca_subject and cert.get_subject(ca_cert)[1] != issuer_ca_subject), None)
+            if ca_subject == None:
+                break
+        return (ca_creds, cred)
     except Exception, e:
         logger.error('Cannot read pkcs7: {1}'.format(e))
         raise Exception('Cannot read pkcs7: {1}'.format(e))
 
+@trace(hide_args=[1, 2], hide_kwargs=['pkcs12_password', 'key_passphrase'])
+def split_pkcs12(pkcs12_certificate, pkcs12_password, key_passphrase):
+    """returns PKCS7 certificate and encrypted private key from a PKCS12 certificate
+    """
+    try:
+        pkcs12_pwd_file = "/root/p12_pwd"
+        if exists(pkcs12_pwd_file):
+            unlink(pkcs12_pwd_file)
+        fh = fdopen(osopen(pkcs12_pwd_file, O_WRONLY|O_CREAT, 0400), 'w')
+        fh.write(pkcs12_password)
+        fh.close()
+        key_pwd_file = "/root/key_pwd"
+        if exists(key_pwd_file):
+            unlink(key_pwd_file)
+        fh = fdopen(osopen(key_pwd_file, O_WRONLY|O_CREAT, 0400), 'w')
+        fh.write(key_passphrase)
+        fh.close()
+        cmd = 'openssl pkcs12 -passin file:{0} -passout file:{1}'.format(pkcs12_pwd_file, key_pwd_file).split()
+        process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
+        process.stdin.write(pkcs12_certificate)
+        ret_code = process.wait()
+        if ret_code != 0:
+            raise Exception("Error in execution 'openssl pkcs12' command")
+        pkcs12_chain = process.stdout.read()
+        logger.debug('PKCS12 command result' + pkcs12_chain)
+        unlink(pkcs12_pwd_file)
+        unlink(key_pwd_file)
+        # isolate private key from pkcs12_chain
+        privkey_pattern = re.compile ( '-----BEGIN ENCRYPTED PRIVATE KEY-----' '.*?' '-----END ENCRYPTED PRIVATE KEY-----', re.DOTALL)
+        private_key = privkey_pattern.findall(pkcs12_chain)[0]
+        # isolate certificates from pkcs12_chain
+        certs_pattern = re.compile ( '-----BEGIN CERTIFICATE-----' '.*?' '-----END CERTIFICATE-----', re.DOTALL)
+        certs_chain = '\n'.join(certs_pattern.findall(pkcs12_chain))
+        # generate pkcs7 certificate
+        fd, tmp_certs_chain_file = mkstemp()
+        fh = file(tmp_certs_chain_file, 'w')
+        fh.write(certs_chain)
+        fh.close()
+        cmd = 'openssl crl2pkcs7 -nocrl -certfile {0}'.format(tmp_certs_chain_file).split()
+        process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
+        ret_code = process.wait()
+        if ret_code != 0:
+            raise Exception("Error in execution 'openssl crl2pkcs7' command")
+        pkcs7_certs = process.stdout.read()
+        unlink(tmp_certs_chain_file)
+        return private_key, pkcs7_certs
+    except Exception, e:
+        raise Exception('Cannot read pkcs12: {0}'.format(e))
+
+
 @trace()
 def extract_crls_from_certifstring(certifstring):
     """Extract crl from certif string
@@ -330,6 +463,12 @@ def extract_crls_from_certifstring(certifstring):
         logger.warning("No crl found in credential")
         pass
 
+@trace()
+def extract_CN_from_certifstring(certifstring):
+    """Extract CommonName from certif string
+    """
+    return cert.get_subject(certifstring)[1]
+
 @trace()
 def ipsec_running():
     """
@@ -345,7 +484,7 @@ def ipsec_running():
 def ipsec_restart():
     """
     """
-    ipsec_restart = "/etc/init.d/ipsec restart > /dev/null"
+    ipsec_restart = "service strongswan restart > /dev/null"
     errcode, output = getstatusoutput(ipsec_restart)
 
 @trace()
