@@ -6,13 +6,16 @@ from os.path import isfile, join
 from commands import getstatusoutput
 from datetime import datetime
 from datetime import timedelta
+from creole import cert
 from arv.web.page import Page, usezephir, IPreferences
 from arv.db.edge import *
 from arv.db.node import *
 from arv.lib.factory import get_object_by_id
-from arv.lib.util import trace, get_req_archive, gen_archive_name, valid
+from arv.lib.util import trace, get_req_archive, gen_archive_name, valid, split_pkcs12
 from arv.lib.logger import logger
-from arv.config import nearly_expired_cred, amon_module_list, sphynx_module_list
+from arv.config import nearly_expired_cred, amon_module_list, \
+                       sphynx_module_list, suffix_dns
+
 from zephir.monitor.agents.rvp import parse_ipsec_statusall
 
 # ____________________________________________________________
@@ -33,9 +36,11 @@ class AddTmplConnect(SavePage):
                 arg='tmplnode2', object_type='tmpl_node')
         credauth = self.get_data_from_request(request=request,
                 arg='credauth', object_type='credential')
+        leftsendcert = self.get_data_from_request(request=request,
+                arg='leftsendcert')
         if not credauth.ca:
             raise Exception('Credential need to be authority')
-        add_tmpl_connect(name, tmpl_node1, tmpl_node2, credauth)
+        add_tmpl_connect(name, tmpl_node1, tmpl_node2, credauth, leftsendcert)
         return "TmplConnect added"
 
 class AddTmplNode(SavePage):
@@ -118,7 +123,7 @@ class AddNode(SavePage):
 class AddConnect(SavePage):
     def render_page(self, request):
         tail_node = self.get_data_from_request(request=request, arg='node1', object_type='node')
-        head_node = self.get_data_from_request(request=request, arg='id', object_type='node')
+        head_node = self.get_data_from_request(request=request, arg='node2', object_type='node')
 
         tail_extr = self.get_data_from_request(request=request, arg='extr1', object_type='extremity')
         head_extr = self.get_data_from_request(request=request, arg='extr2', object_type='extremity')
@@ -126,11 +131,13 @@ class AddConnect(SavePage):
         tail_cred = self.get_data_from_request(request=request, arg='cred1', object_type='credential')
         head_cred = self.get_data_from_request(request=request, arg='cred2', object_type='credential')
 
+        leftsendcert = self.get_data_from_request(request=request, arg='leftsendcert')
+
         tmpl_connect = self.get_data_from_request(request=request, arg='tmpl_connect', object_type='tmpl_connect')
         edges = self.get_data_from_request(request=request, arg='edges', load_json=True)
 
         connect = tmpl_connect.add_connect(tail_node, head_node,
-                tail_extr, head_extr, tail_cred, head_cred)
+                tail_extr, head_extr, tail_cred, head_cred, leftsendcert)
         for tmpledgeid, activated in edges.items():
             if activated:
                 tmpl_edge = get_object_by_id(tmpledgeid, 'tmpl_edge')
@@ -146,9 +153,23 @@ class AddImportedCred(SavePage):
         node.import_credential(private_key=private_key, credential=public_key, passwd=password)
         return 'Credential imported'
 
+class AddImportedPKCS12(SavePage):
+    def render_page(self, request):
+        pkcs12_certificate = self.get_data_from_request(request=request, arg='pkcs12_file')
+        pkcs12_password = self.get_data_from_request(request=request, arg='pkcs12_password', required=True)
+        key_passphrase = self.get_data_from_request(request=request, arg='key_passphrase', required=True)
+        node = self.get_data_from_request(request=request, arg='node', object_type='node')
+        private_key , pkcs7_certificate = split_pkcs12(pkcs12_certificate, pkcs12_password, key_passphrase)
+        node.import_credential(private_key=private_key, credential=pkcs7_certificate, passwd=key_passphrase)
+        return 'Credential imported'
+
 class AddGeneratedCred(SavePage):
     def render_page(self, request):
         name = self.get_data_from_request(request=request, arg='name')
+        if suffix_dns != "":
+            splitted_name = name.split(suffix_dns)
+            if len(splitted_name) == 1 or len(splitted_name) == 2 and splitted_name[-1] != "":
+                name = name + suffix_dns
         passwd = self.get_data_from_request(request=request, arg='passwd')
         node = self.get_data_from_request(request=request, arg='node', object_type='node')
         #cred_priv_type = self.get_data_from_request(request=request, arg='cred_priv_type')
@@ -170,7 +191,8 @@ class DlCredential(SavePage):
 class AddCredAuth(SavePage):
     def render_page(self, request):
         credentialauth = self.get_data_from_request(request=request, arg='credentialauth')
-        add_credential_auth(credential=credentialauth)
+        if add_certificate_authorities(certif=credentialauth) is None:
+            raise Exception("No CA certificate added")
         return 'Credential auth added'
 
 class AddExtremity(SavePage):
@@ -251,6 +273,7 @@ class ModConnect(SavePage):
         head_extr = self.get_data_from_request(request=request, arg='extr2', object_type='extremity')
         tail_cred = self.get_data_from_request(request=request, arg='cred1', object_type='credential')
         head_cred = self.get_data_from_request(request=request, arg='cred2', object_type='credential')
+        leftsendcert = self.get_data_from_request(request=request, arg='leftsendcert')
         edges = self.get_data_from_request(request=request, arg='edges', load_json=True)
 
         if connect.tail_node == tail_extr.node:
@@ -263,7 +286,7 @@ class ModConnect(SavePage):
             connect.mod_head_extr(tail_extr)
             connect.mod_tail_cred(head_cred)
             connect.mod_head_cred(tail_cred)
-
+        connect.leftsendcert = leftsendcert
 
         current_edges = {}
         for edge in connect.edges:
@@ -394,20 +417,22 @@ class ModExtrIP(SavePage):
 # ____________________________________________________________
 # get pages
 class GetNodes(Page):
+    @trace()
     def render_page(self, request):
         ret = []
         connected = self.get_data_from_request(request=request, arg='connected', required=False)
         if connected == 'true':
-            nodea = self.get_data_from_request(request=request, arg='id', object_type='node')
-            nodes = get_connected_nodes(nodea)
+            rvp_node1 = self.get_data_from_request(request=request, arg='id', object_type='node')
+            nodes = get_connected_nodes(rvp_node1)
         else:
-            nodea = self.get_data_from_request(request=request, arg='id', object_type='node', required=False)
-            nodes = get_nodes(nodea)
+            rvp_node1 = self.get_data_from_request(request=request, arg='id', object_type='node', required=False)
+            nodes = get_nodes(rvp_node1)
         for node in nodes:
             ret.append({'uai': node.uai, 'node': node.name, 'id': node.id})
         return ret
 
 class GetNodesState(Page):
+    @trace()
     def render_page(self, request):
         ret = []
         nodes = get_nodes()
@@ -431,7 +456,7 @@ class GetNodesState(Page):
             down = 0
 
         tmpl_nodes = get_tmpl_nodes(u'sphynx')
-        current_node = tmpl_nodes[0].nodes[0]
+        current_node = [] if tmpl_nodes[0].nodes == [] else tmpl_nodes[0].nodes[0]
         for node in nodes:
             if node == current_node:
                 state = ""
@@ -479,21 +504,38 @@ class GetNodesState(Page):
 class GetEdges(Page):
     def render_page(self, request):
         ret = []
-        nodea = self.get_data_from_request(request=request, arg='nodea', object_type='node')
-        nodeb = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
-        for connect in get_connects(nodea, nodeb):
+        rvp_node1 = self.get_data_from_request(request=request, arg='nodea', object_type='node')
+        rvp_node2 = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
+        logger.debug("Column 1 node : {0}".format(str(rvp_node1.name)))
+        logger.debug("Column 2 node : {0}".format(str(rvp_node2.name)))
+        for connect in get_connects(rvp_node1, rvp_node2):
             if connect.edges == []:
                 ret.append({'connect': connect.tmpl_connect.name})
             else:
+                # Flag for the ordering network view into a tunnel
+                reverse_view = False
                 connect_infos = connect.tmpl_connect.name
-                if connect.head_extr.node == nodea:
+                if connect.head_extr.node == rvp_node1:
+                    # Selected head node in column 1
+                    logger.debug("Ordered view, head node : {0}".format(str(rvp_node1.name)))
                     head_extr = connect.head_extr
-                elif connect.head_extr.node == nodeb:
+                    reverse_view = False
+                elif connect.head_extr.node == rvp_node2:
+                    # Selected head node in column 2
+                    logger.debug("Reverse view, head node : {0}".format(str(rvp_node2.name)))
                     tail_extr = connect.head_extr
-                if connect.tail_extr.node == nodea:
+                    reverse_view = True
+                if connect.tail_extr.node == rvp_node1:
+                    # Selected tail node in column 1
+                    logger.debug("Reverse view, tail node : {0}".format(str(rvp_node1.name)))
                     head_extr = connect.tail_extr
-                elif connect.tail_extr.node == nodeb:
+                    reverse_view = True
+                elif connect.tail_extr.node == rvp_node2:
+                    # Selected tail node in column 2
+                    logger.debug("Ordered view, tail node : {0}".format(str(rvp_node2.name)))
                     tail_extr = connect.tail_extr
+                    reverse_view = False
+                # Mapping to the connect info treeview
                 if head_extr.pub_ip == head_extr.priv_ip:
                     connect_infos += " - {0}".format(str(head_extr.pub_ip))
                 else:
@@ -502,60 +544,67 @@ class GetEdges(Page):
                     connect_infos += " ... {0}".format(str(tail_extr.pub_ip))
                 else:
                     connect_infos += " ... {0} <-- {1}".format(str(tail_extr.priv_ip), str(tail_extr.pub_ip))
+                connect_infos += " / " + " certificate send : " + connect.leftsendcert.upper() + " via ipsec protocol"
+                # extracts edges for the connect
                 for edge in connect.edges:
-                    if edge.tmpl_edge.tmpl_vertex_a.tmpl_node == nodea.tmpl_node:
-                        vertexa = edge.tmpl_edge.tmpl_vertex_a.name
-                        vertexb = edge.tmpl_edge.tmpl_vertex_b.name
+                    logger.debug(u"Edge : {0}".format(edge.tmpl_edge.name))
+                    if connect.head_node.tmpl_node == connect.tail_node.tmpl_node:
+                        if not reverse_view:
+                            # Ordering the vertices into columns to be displayed
+                            column1_vertex = edge.tmpl_edge.tmpl_vertex_a
+                            column2_vertex = edge.tmpl_edge.tmpl_vertex_b
+                        else:
+                            column1_vertex = edge.tmpl_edge.tmpl_vertex_b
+                            column2_vertex = edge.tmpl_edge.tmpl_vertex_a
                     else:
-                        vertexa = edge.tmpl_edge.tmpl_vertex_b.name
-                        vertexb = edge.tmpl_edge.tmpl_vertex_a.name
-                    for vertex in edge.tmpl_edge.tmpl_vertex_a.vertices:
-                        if vertex.node == nodea:
+                        if edge.tmpl_edge.tmpl_vertex_a.tmpl_node == rvp_node1.tmpl_node:
+                            column1_vertex = edge.tmpl_edge.tmpl_vertex_a
+                            column2_vertex = edge.tmpl_edge.tmpl_vertex_b
+                        else:
+                            column1_vertex = edge.tmpl_edge.tmpl_vertex_b
+                            column2_vertex = edge.tmpl_edge.tmpl_vertex_a
+
+                    column1_vertex_infos = column1_vertex.name
+                    column2_vertex_infos = column2_vertex.name
+                    logger.debug("Column 1 tmplvertex : {0}".format(str(column1_vertex_infos)))
+                    logger.debug("Column 2 tmplvertex : {0}".format(str(column2_vertex_infos)))
+                    # extracts vertices for column 1
+                    for vertex in column1_vertex.vertices:
+                        logger.debug("Column 1 vertex loop : {0}".format(str(vertex.tmpl_vertex.name)))
+                        if vertex.node == rvp_node1:
+                            logger.debug("Match to column 1 node")
                             if vertex.tmpl_vertex.mimetype == u'network':
                                 sep = " / "
                             elif vertex.tmpl_vertex.mimetype == u'ip':
                                 sep = ""
                             elif vertex.tmpl_vertex.mimetype == u'range':
                                 sep = " ... "
-                            vertexa += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
-                        elif vertex.node == nodeb:
+                            column1_vertex_infos += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
+                    # extracts vertices for column 2
+                    for vertex in column2_vertex.vertices:
+                        logger.debug("Column 2 vertex loop : {0}".format(str(vertex.tmpl_vertex.name)))
+                        if vertex.node == rvp_node2:
+                            logger.debug("Match to column 2 node")
                             if vertex.tmpl_vertex.mimetype == u'network':
                                 sep = " / "
                             elif vertex.tmpl_vertex.mimetype == u'ip':
                                 sep = ""
                             elif vertex.tmpl_vertex.mimetype == u'range':
                                 sep = " ... "
-                            vertexb += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
-                    for vertex in edge.tmpl_edge.tmpl_vertex_b.vertices:
-                        if vertex.node == nodea:
-                            if vertex.tmpl_vertex.mimetype == u'network':
-                                sep = " / "
-                            elif vertex.tmpl_vertex.mimetype == u'ip':
-                                sep = ""
-                            elif vertex.tmpl_vertex.mimetype == u'range':
-                                sep = " ... "
-                            vertexa += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
-                        elif vertex.node == nodeb:
-                            if vertex.tmpl_vertex.mimetype == u'network':
-                                sep = " / "
-                            elif vertex.tmpl_vertex.mimetype == u'ip':
-                                sep = ""
-                            elif vertex.tmpl_vertex.mimetype == u'range':
-                                sep = " ... "
-                            vertexb += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
+                            column2_vertex_infos += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
                     ret.append({'edge': edge.tmpl_edge.name, 'id': edge.id,
                             'connect': connect_infos,
-                            'vertexa': vertexa , 'vertexb': vertexb,
+                            'vertexa': column1_vertex_infos , 'vertexb': column2_vertex_infos,
                             'activate': False})
         return ret
 
 class GetConnects(Page):
     def render_page(self, request):
         ret = []
-        nodea = self.get_data_from_request(request=request, arg='nodea', object_type='node')
-        nodeb = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
-        for connect in get_connects(nodea, nodeb):
-            ret.append({'id': connect.id, 'connect': connect.tmpl_connect.name})
+        rvp_node1 = self.get_data_from_request(request=request, arg='nodea', object_type='node')
+        rvp_node2 = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
+        for connect in get_connects(rvp_node1, rvp_node2):
+            ret.append({'id': connect.id, 'connect': connect.tmpl_connect.name, 'leftsendcert': connect.leftsendcert})
         return ret
 
 class GetTmplNodes(Page):
@@ -587,10 +636,12 @@ class GetTmplConnects(Page):
             if tmpl_connect not in tmpl_connects:
                 ret.append({'tmplconnect': tmpl_connect.name,
                         'id': tmpl_connect.id,
+                        'ca_cert': tmpl_connect.cred_auth.name,
                         'tmplnodea': tmpl_connect.tmpl_node_a.name,
                         'tmplnodeaid': tmpl_connect.tmpl_node_a.id,
                         'tmplnodeb': tmpl_connect.tmpl_node_b.name,
-                        'tmplnodebid': tmpl_connect.tmpl_node_b.id})
+                        'tmplnodebid': tmpl_connect.tmpl_node_b.id,
+                        'leftsendcert': tmpl_connect.leftsendcert})
         return ret
 
 class GetTmplEdges(Page):
@@ -627,73 +678,82 @@ class GetTmplVertices(Page):
         name = self.get_data_from_request(request=request, arg='name', required=False)
         force_zephir = self.get_data_from_request(request=request, arg='force_zephir', required=False)
         modtmplvertex = self.get_data_from_request(request=request, arg='modtmplvertex', required=False)
-        if None not in [tmpl_node, node] or tmpl_node == node:
-            raise Exception('Must specified tmpl_node or node')
+        #if None not in [tmpl_node, node] or tmpl_node == node:
+        #    raise Exception('Must specified tmpl_node or node')
         if node != None:
             tmpl_node = node.tmpl_node
             tmpl_vertex = {}
             for vertex in node.vertices:
                 tmpl_vertex[vertex.tmpl_vertex] = (vertex.ip1, vertex.ip2)
-        for tmplvertex in get_tmpl_vertices(tmpl_node):
-            ip1 = ''
-            ip2 = ''
-            module = ''
-            if node != None and force_zephir != 'true':
-                # Charge les valeurs des vertex d'un node
-                ip1, ip2 = tmpl_vertex.get(tmplvertex, ('', ''))
-            else:
-                if uai != None:
-                    # Recharge les infos Zéphir et tmplvertex
-                    prefs = request.getSession(IPreferences)
-                    if prefs.zephir != None:
-                        # Si connecté avec user Zéphir
-                        if tmplvertex.zephir_module:
-                            # Recharge les valeur depuis Zéphir
-                            ip1 = prefs.zephir.get_var(
-                                    uai=uai,
-                                    name=name,
-                                    module=tmplvertex.zephir_module,
-                                    var=tmplvertex.zephir_var_ip1
-                            )
-                            if tmplvertex.mimetype != u'ip':
-                                ip2 = prefs.zephir.get_var(
+        if tmpl_node != None:
+            for tmplvertex in get_tmpl_vertices(tmpl_node):
+                ip1 = ''
+                ip2 = ''
+                module = ''
+                if node != None and force_zephir != 'true':
+                    # Charge les valeurs des vertex d'un node
+                    ip1, ip2 = tmpl_vertex.get(tmplvertex, ('', ''))
+                else:
+                    if uai != None:
+                        # Recharge les infos Zéphir et tmplvertex
+                        prefs = request.getSession(IPreferences)
+                        if prefs.zephir != None:
+                            # Si connecté avec user Zéphir
+                            if tmplvertex.zephir_module:
+                                # Recharge les valeur depuis Zéphir
+                                ip1 = prefs.zephir.get_var(
                                         uai=uai,
                                         name=name,
                                         module=tmplvertex.zephir_module,
-                                        var=tmplvertex.zephir_var_ip2)
+                                        var=tmplvertex.zephir_var_ip1
+                                )
+                                if tmplvertex.mimetype != u'ip':
+                                    ip2 = prefs.zephir.get_var(
+                                            uai=uai,
+                                            name=name,
+                                            module=tmplvertex.zephir_module,
+                                            var=tmplvertex.zephir_var_ip2)
+                            else:
+                                # Recharge les valeur depuis tmplvertex
+                                ip1 = tmplvertex.zephir_var_ip1
+                                ip2 = tmplvertex.zephir_var_ip2
                         else:
-                            # Recharge les valeur depuis tmplvertex
-                            ip1 = tmplvertex.zephir_var_ip1
-                            ip2 = tmplvertex.zephir_var_ip2
+                            # Si connecté avec user system
+                            if not tmplvertex.zephir_module:
+                                ip1 = tmplvertex.zephir_var_ip1
+                                ip2 = tmplvertex.zephir_var_ip2
                     else:
-                        # Si connecté avec user system
-                        if not tmplvertex.zephir_module:
+                        #if search tmplVertex attribute
+                        module = tmplvertex.zephir_module
+                        if modtmplvertex == 'true' or not module:
+                            # Si modif tmplvertex
+                            # ou si ajout manuel serveur
                             ip1 = tmplvertex.zephir_var_ip1
                             ip2 = tmplvertex.zephir_var_ip2
-                else:
-                    #if search tmplVertex attribute
-                    module = tmplvertex.zephir_module
-                    if modtmplvertex == 'true':
-                    # Si modif tmplvertex
-                        ip1 = tmplvertex.zephir_var_ip1
-                        ip2 = tmplvertex.zephir_var_ip2
-                    elif not tmplvertex.zephir_module:
-                    # Si ajout manuel serveur
-                        ip1 = tmplvertex.zephir_var_ip1
-                        ip2 = tmplvertex.zephir_var_ip2
 
-                if (ip1, ip2) in [('', ''), (None, None)] and \
-                        force_zephir == 'true':
-                    #if force_zephir is set and no value from Zéphir
-                    ip1, ip2 = tmpl_vertex.get(tmplvertex, ('', ''))
+                    if (ip1, ip2) in [('', ''), (None, None)] and \
+                            force_zephir == 'true':
+                        #if force_zephir is set and no value from Zéphir
+                        ip1, ip2 = tmpl_vertex.get(tmplvertex, ('', ''))
 
-            ret.append({'tmplvertex': tmplvertex.name,
-                'id': tmplvertex.id,
-                'mimetype': tmplvertex.mimetype,
-                'module': module,
-                'ip1': ip1,
-                'ip2': ip2
-            })
+                ret.append({'tmplvertex': tmplvertex.name,
+                    'id': tmplvertex.id,
+                    'mimetype': tmplvertex.mimetype,
+                    'module': module,
+                    'ip1': ip1,
+                    'ip2': ip2
+                })
+        else:
+            for tmplvertex in get_tmpl_vertices():
+                ret.append({'tmplvertexname': tmplvertex.name,
+                    'id': tmplvertex.id,
+                    'tmplvertexmimetype': tmplvertex.mimetype,
+                    'tmplnode': tmplvertex.tmpl_node.name,
+                    'tmplnodeid': tmplvertex.tmpl_node.id,
+                    'tmplvertexmodule': tmplvertex.zephir_module,
+                    'tmplvertexip1': tmplvertex.zephir_var_ip1,
+                    'tmplvertexip2': tmplvertex.zephir_var_ip2,
+                })
 
         return ret
 
@@ -758,11 +818,13 @@ class GetAllCred(Page):
                 related_node_name = uai + " - " + name
             elif not cred.ca:
                 related_node_name = "PAS DE SERVEUR RVP ASSOCIÉ !!!"
+            issuer = cert.get_issuer_subject(cred.credential)[1]
             ret.append({'id':cred.id, 'name': cred.name,
                 'ca': cred.ca, 'expiration_date': cred.expiration_date,
                 'delta': delta, 'is_expired': is_expired,
                 'is_nearly_expired': is_nearly_expired,
-                'node': related_node_name})
+                'node': related_node_name,
+                'issuer': issuer})
         ret.sort(sort_cred)
         return ret
 
