@@ -16,6 +16,7 @@ ARV.addnode2 = (function() {
         },[
             {name : 'connect'},
             {name : 'id'},
+            {name : 'leftsendcert'},
         ]),
         url: 'arv/get_connects',
     });
@@ -23,7 +24,8 @@ ARV.addnode2 = (function() {
         store: connectstore,
         columns: [
             {header: Ext.i18n.connect_window, sortable: true, dataIndex: 'connect'},
-            {header: 'ID', sortable: true, dataIndex: 'id', hidden: true}
+            {header: 'ID', sortable: true, dataIndex: 'id', hidden: true},
+            {header: 'leftsendcert', sortable: true, dataIndex: 'leftsendcert', hidden: true}
         ],
         deleteUrl: 'arv/del_connect',
         btnAddEvent: 'addmodconnect',
@@ -44,7 +46,7 @@ ARV.addnode2 = (function() {
     });
 
     return {
-        openWindow: function(node_a_id, node_a_name, node_b_id, node_b_name, connect_id){
+        openWindow: function(node_a_id, node_a_name, node_b_id, node_b_name, connect_id, leftsendcert){
             var create_extrwindow = function(id) {
                 form = {
                     xtype: 'form',
@@ -88,6 +90,18 @@ ARV.addnode2 = (function() {
                 }).show();
             };
 
+            var nodes_store = new Ext.data.Store({
+                reader: new Ext.data.JsonReader({
+                    root: 'data',
+                    successProperty: 'success',
+                },[
+                    {name : 'id'},
+                    {name : 'uai'},
+                    {name : 'node'}
+                ]),
+                url: 'arv/get_nodes'
+            });
+
             var edges_store = new Ext.data.Store({
                 reader: new Ext.data.JsonReader({
                     root: 'data',
@@ -101,17 +115,6 @@ ARV.addnode2 = (function() {
                 url: 'arv/get_tmpl_edges'
             });
 
-            //var tmplconnect_store = new Ext.data.Store({
-            //    reader: new Ext.data.JsonReader({
-            //        root :'data',
-            //        successProperty: 'success',
-            //    },[
-            //        {name : 'tmplconnect'},
-            //        {name : 'id'},
-            //    ]),
-            //    url: 'arv/get_tmpl_connects'
-            //});
-
             Ext.apply(Ext.form.VTypes, {
                 IPAddress:  function(v) {
                     return /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/.test(v);
@@ -130,30 +133,48 @@ ARV.addnode2 = (function() {
                 width:40
             });
 
+            var leftsendcert_choice = new Ext.data.ArrayStore({
+                fields: ['id', 'leftsendcertval'],
+                data: [['always', Ext.i18n.leftsendcert_always2], ['never', Ext.i18n.leftsendcert_never2]]
+            });
+
+            var filter_grid = new Ext.ux.grid.GridFilters({
+                // encode and local configuration options defined previously for easier reuse
+                // encode: encode, // json encode the filter query
+                local: true,   // defaults to false (remote filtering)
+                filters: [{
+                    type: 'string',
+                    dataIndex: 'uai',
+                }, {
+                    type: 'string',
+                    dataIndex: 'node',
+                }]
+            });
+
             var cardcontent = [{
                 id: 'card-0',
-                xtype: 'form',
-                border: false,
-                items: [{
-                    xtype: "arvcombo",
-                    id: 'node2',
-                    fieldLabel: Ext.i18n.addnode2_node,
-                    record: [
-                        {name : 'node'},
-                        {name : 'id'},
+                xtype: 'grid',
+                border: true,
+                layout: 'fit',
+                plugins: filter_grid,
+                store: nodes_store,
+                colModel: new Ext.grid.ColumnModel({
+                    columns: [
+                        {header: 'id', sortable: true, dataIndex: 'id', hidden: true},
+                        {id: 'uai', header: Ext.i18n.uai, sortable: true, dataIndex: 'uai', width: '100px'},
+                        {id: 'node', header: Ext.i18n.node, sortable: true, dataIndex: 'node', width: '300px'},
                     ],
-                    baseParams: {id: node_a_id},
-                    url: 'arv/get_nodes',
-                    autoLoad: false,
-                    displayField: 'node',
-                    listen: {
-                        'load': function(store, records, option){
-                            if (records.length != 0) {
-                                Ext.getCmp('node2').setValue(records[0].get('id'));
-                            }
-                        }
-                    }
-                }],
+                    defaults: {
+                        sortable: true,
+                    },
+                    plugins: filter_grid
+                }),
+                //singleSelect
+                sm: new Ext.grid.RowSelectionModel({
+                    singleSelect:true,
+                }),
+                enableColumnHide: false,
+
             },{
                 id: 'card-1',
                 xtype: 'form',
@@ -164,6 +185,7 @@ ARV.addnode2 = (function() {
                     record: [
                         {name : 'tmplconnect'},
                         {name : 'id'},
+                        {name : 'leftsendcert'},
                     ],
                     url: 'arv/get_tmpl_connects',
                     autoLoad: false,
@@ -373,13 +395,41 @@ ARV.addnode2 = (function() {
                             text: Ext.i18n.addnode2_choice_button,
                             anchor:'95%',
                             handler: function () {
+                                if (node_b_id == null) {
+                                    node2 = Ext.getCmp('card-0');
+                                    node2id = node2.getSelectionModel().getSelected().get('id');
+                                } else {
+                                    node2id = node_b_id;
+                                }
                                 ARV.modextip.openIPWindow(
-                                    Ext.getCmp('node2').getValue(),
+                                    node2id,
                                     'extrip_node2_changed'
                                 );
                             }
                         }]
                     }]
+                }, {
+                    border: false,
+                    layout: 'form',
+                    items: [{
+                        xtype: 'combo',
+                        name: 'leftsendcert',
+                        id: 'leftsendcert_combo_connect',
+                        fieldLabel: Ext.i18n.leftsendcert,
+                        store: leftsendcert_choice,
+                        displayField: 'leftsendcertval',
+                        width: 226,
+                        editable: false,
+                        mode: 'local',
+                        value: leftsendcert,
+                        hiddenName:'id',
+                        valueField: 'id',
+                        allowBlank: false,
+                        typeAhead: true,
+                        selectOnFocus: true,
+                        forceSelection: true,
+                        triggerAction: 'all',
+                    }]
                 }]
             }, {
                 id: 'card-3',
@@ -419,11 +469,13 @@ ARV.addnode2 = (function() {
                     break;
                     case 1:
                         //choisir un modèle de lien sécurisé
-                        idnode2 = Ext.getCmp('node2').getValue();
-                        if (idnode2 == '') {
+                        node2 = Ext.getCmp('card-0');
+                        if (! node2.getSelectionModel().hasSelection()) {
                             error_msg(Ext.i18n.field_empty)
                             return;
                         };
+                        node2 = Ext.getCmp('card-0');
+                        idnode2 = node2.getSelectionModel().getSelected().get('id');
                         Ext.getCmp('button-addnode2').disable();
                         tmplconnect_store = Ext.getCmp('tmplconnect').store;
                         tmplconnect_store.removeAll();
@@ -444,20 +496,28 @@ ARV.addnode2 = (function() {
                             };
                         } else {
                             idtmplconnect = null;
+                        };
+                        if (node_b_id == null) {
+                            node2 = Ext.getCmp('card-0');
+                            node2_name = node2.getSelectionModel().getSelected().get('node');
+                            node2id = node2.getSelectionModel().getSelected().get('id');
+                        } else {
+                            node2id = node_b_id;
+                            node2_name = node_b_name;
                         }
-                        Ext.getCmp('node2_label').setText('<b>'+Ext.getCmp('node2').getRawValue()+'</b>', false);
+                        Ext.getCmp('node2_label').setText('<b>'+node2_name+'</b>', false);
                         extr1_store = Ext.getCmp('extr1').store;
                         extr1_store.removeAll();
                         if (connect_id == null) {
                             extr1_store.baseParams={node: node_a_id};
                         } else {
                             extr1_store.baseParams={node: node_a_id, connect_id: connect_id};
-                        }
+                        };
                         extr1_store.load();
                         extr2_store = Ext.getCmp('extr2').store;
                         extr2_store.removeAll();
                         if (connect_id == null) {
-                            extr2_store.baseParams={node: Ext.getCmp('node2').getValue()};
+                            extr2_store.baseParams={node: node2id};
                         } else {
                             extr2_store.baseParams={node: node_b_id, connect_id: connect_id};
                         }
@@ -482,16 +542,27 @@ ARV.addnode2 = (function() {
                         cred2_store.removeAll();
                         if (connect_id == null) {
                             cred2_store.baseParams={
-                                node: Ext.getCmp('node2').getValue(),
+                                node: node2id,
                                 tmplconnect_id: idtmplconnect
                             }
                         } else {
                             cred2_store.baseParams={
-                                node: Ext.getCmp('node2').getValue(),
+                                node: node2id,
                                 connect_id: connect_id
                             }
                         };
                         cred2_store.load();
+                        if (connect_id == null) {
+                            tmplconnect_store = Ext.getCmp('tmplconnect').store;
+                            tmplconnect_id = Ext.getCmp('tmplconnect').getValue();
+                            tmplconnect_store.each(
+                                function (record){
+                                    if (record.get('id') == tmplconnect_id)
+                                        leftsendcert = record.get('leftsendcert')
+                                }
+                          );
+                        };
+                        Ext.getCmp('leftsendcert_combo_connect').setValue(leftsendcert);
                     break;
                     case 3:
                         //modèle de réseau -> modèle de réseau
@@ -534,25 +605,31 @@ ARV.addnode2 = (function() {
                     text: Ext.i18n.save,
                     disabled: true,
                     handler: function() {
-                        idnode2 = Ext.getCmp('node2').getValue();
+                        if (node_b_id == null) {
+                            node2 = Ext.getCmp('card-0');
+                            idnode2 = node2.getSelectionModel().getSelected().get('id');
+                        } else {
+                            idnode2 = node_b_id;
+                        }
                         var records = {};
                         edges_store.each(
                             function (record){
                                 records[record.get('id')] = record.get('activate');
                             }
                         );
-                        Ext.getCmp('card-0').getForm().submit({
+                        Ext.getCmp('card-1').getForm().submit({
                             url: 'arv/add_connect',
                             //waitMsg: Ext.i18n.dialog_submit,
                             params: {
                                 node1: node_a_id,
+                                node2: idnode2,
                                 tmpl_connect: Ext.getCmp('tmplconnect').getValue(),
                                 edges: Ext.util.JSON.encode(records),
                                 extr1: Ext.getCmp('extr1').getValue(),
                                 extr2: Ext.getCmp('extr2').getValue(),
                                 cred1: Ext.getCmp('cred1').getValue(),
-                                cred2: Ext.getCmp('cred2').getValue()
-
+                                cred2: Ext.getCmp('cred2').getValue(),
+                                leftsendcert: Ext.getCmp('leftsendcert_combo_connect').getValue()
                             },
                             success: response_success,
                         })
@@ -567,24 +644,30 @@ ARV.addnode2 = (function() {
                     text: Ext.i18n.modify,
                     disabled: true,
                     handler: function() {
-                        idnode2 = Ext.getCmp('node2').getValue();
+                        if (node_b_id == null) {
+                            node2 = Ext.getCmp('card-0');
+                            idnode2 = node2.getSelectionModel().getSelected().get('id');
+                        } else {
+                            idnode2 = node_b_id;
+                        }
                         var records = {};
                         edges_store.each(
                             function (record){
                                 records[record.get('id')] = record.get('activate');
                             }
                         );
-                        Ext.getCmp('card-0').getForm().submit({
+                        Ext.getCmp('card-1').getForm().submit({
                             url: 'arv/mod_connect',
                             //waitMsg: Ext.i18n.dialog_submit,
                             params: {
+                                id: idnode2,
                                 connect_id: connect_id,
                                 edges: Ext.util.JSON.encode(records),
                                 extr1: Ext.getCmp('extr1').getValue(),
                                 extr2: Ext.getCmp('extr2').getValue(),
                                 cred1: Ext.getCmp('cred1').getValue(),
-                                cred2: Ext.getCmp('cred2').getValue()
-
+                                cred2: Ext.getCmp('cred2').getValue(),
+                                leftsendcert: Ext.getCmp('leftsendcert_combo_connect').getValue()
                             },
                             success: response_success,
                         })
@@ -635,12 +718,11 @@ ARV.addnode2 = (function() {
             }).show();
 
             if (node_b_id != null) {
-                Ext.getCmp('node2').setValue(node_b_id);
-                Ext.getCmp('node2').setRawValue(node_b_name);
-                //Ext.getCmp('tmplconnect').setValue(pouet);
+                Ext.getCmp('tmplconnect').setValue(connect_id);
+                Ext.getCmp('tmplconnect').setRawValue(connect_id);
                 navHandler(0);
             } else {
-                Ext.getCmp('node2').store.load();
+                nodes_store.load();
             }
         },
         openConnectWindow: function(nodea, nodeb){
@@ -651,10 +733,10 @@ ARV.addnode2 = (function() {
         },
         load: function(nodea, nodeb) {
             connectstore.removeAll();
-			if (nodea != null && nodeb != null) {
-	            connectstore.baseParams={nodea: nodea, nodeb: nodeb};
-    	        connectstore.load();
-			};
+            if (nodea != null && nodeb != null) {
+                connectstore.baseParams={nodea: nodea, nodeb: nodeb};
+                connectstore.load();
+            };
         },
         loadIP: function() {
             Ext.getCmp('extr1').store.load();
@@ -667,6 +749,13 @@ ARV.addnode2 = (function() {
                 return null
             }
         },
+        getSelectedLeftSendCert: function() {
+            if (connect_grid.getSelectionModel().hasSelection()) {
+                return connect_grid.getSelectionModel().getSelected().get('leftsendcert');
+            } else {
+                return null
+            }
+        },
     };
 })();
 
